/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2005 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANDYNAMICILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "kz-actions-dynamic.h"

#include <string.h>
#include <glib/gi18n.h>
#include "gtk-utils.h"
#include "kazehakase.h"
#include "kz-icons.h"

#define KZ_ACTIONS_DYNAMIC_COPY_IN_USER_FORMAT_KEY "KzActionsDynamic::CopyInUserFormat"


static void
cb_copy_in_user_format_preference_activate (GtkWidget *menuitem, KzWindow *kz)
{
	GtkAction *action;

	g_return_if_fail(KZ_IS_WINDOW(kz));
	
	action = gtk_action_group_get_action(kz->actions, "ShowCopyFormat");
	gtk_action_activate(action);
}

static void
cb_copy_in_user_format_menuitem_activate (GtkWidget *menuitem, KzTabLabel *kztab)
{
	KzEmbed *kzembed;
	gint idx = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem),
						     KZ_ACTIONS_DYNAMIC_COPY_IN_USER_FORMAT_KEY));
	gchar conf_key[] = "copy_document_format99";
	gchar *format;
	gchar *text;
	gchar *str;
	gint i;
	gint counts = 0;

	g_return_if_fail(0 < idx && idx < 100);
	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));

	kzembed = kztab->kzembed;
	 if (!KZ_IS_EMBED(kzembed)) return;

	text = kz_embed_get_selection_string(kzembed);
	if (!text) text = g_strdup("");

	g_snprintf(conf_key, sizeof(conf_key), "copy_document_format%d", idx);
	format = KZ_CONF_GET_STR("Global" , conf_key);
	g_return_if_fail(format);

	for(i = 0; format[i] != '\0'; i++)
		if(format[i] == '%') counts++;
	counts++;

	if(counts != 1)
	{
		gchar **temp = g_alloca(sizeof(gchar*) * (counts + 1));
		gchar **splited = g_strsplit(format , "%" , counts);
		gint st = 0;

		for(i = 0; i < counts-1; i++)
		{
			if(st == 0)
			{
				/* first state */
				temp[i] = splited[i];
				st = 1;
			}
			else if (st == 2)
			{
				/* after %t% and so on */
				temp[i] = splited[i];
				st = 1;
			}
			else if (splited[i][0] == '\0')
			{
				/* "%%" */
				temp[i] = "%";
				st = 0;
			}
			else if (splited[i][1] != '\0')
			{
				/* %text% */
				temp[i] = splited[i];
				st = 1;
			}
			else if(splited[i][0] == 't')
			{
				/* %t% */
				temp[i] = (gchar*)kz_embed_get_title(kzembed);
				st = 2;
			}
			else if(splited[i][0] == 'u')
			{
				/* %u% */
				temp[i] = (gchar*)kz_embed_get_location(kzembed);
				st = 2;
			}
			else if(splited[i][0] == 's')
			{
				/* %s% */
				temp[i] = text;
				st = 2;
			}
			else
			{
				/* '%' + other char + '%' */
				temp[i] = splited[i];
				st = 1;
			}
		}
		temp[counts-1] = splited[counts-1];
		temp[counts] = NULL;

		str = g_strjoinv(NULL , temp);
		g_strfreev(splited);
	}
	else
	{
		str = g_strdup(format);
	}

	g_free(format);

	gtkutil_copy_text(str);

	if(str) g_free(str);
	if(text) g_free(text);
}

void
kz_actions_dynamic_append_copy_in_user_format_menuitem (KzTabLabel *kztab,
						        GtkMenuItem *menu)
{
	GtkWidget *copy_in_user_format_submenu = NULL;
	GtkWidget *menuitem;
	gchar *title;
	gchar conf_key[] = "copy_document_format_title99";
	gint i;

	copy_in_user_format_submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menu));
	if (copy_in_user_format_submenu)
	{
		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu));
	}

	copy_in_user_format_submenu = gtk_menu_new();
	gtk_widget_show(copy_in_user_format_submenu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu),
				  copy_in_user_format_submenu);

	/* append copy in user format menuitem */
	menuitem = gtk_menu_item_new_with_mnemonic(_("_Preference..."));
	g_signal_connect(menuitem, "activate",
			 G_CALLBACK(cb_copy_in_user_format_preference_activate),
			 kztab->kz);

	gtk_menu_shell_append(GTK_MENU_SHELL(copy_in_user_format_submenu),
			      menuitem);
	gtk_widget_show(menuitem);
	
	menuitem = gtk_separator_menu_item_new();
	gtk_menu_shell_append(GTK_MENU_SHELL(copy_in_user_format_submenu),
			      menuitem);
	gtk_widget_show(menuitem);

	for (i = 1; i < 100; i++)
	{
		g_snprintf(conf_key, sizeof(conf_key), "copy_document_format_title%d", i);
		title = KZ_CONF_GET_STR("Global" , conf_key);
		if(!title) break;

		menuitem = gtk_menu_item_new_with_label(title);

		g_object_set_data (G_OBJECT(menuitem),
				   KZ_ACTIONS_DYNAMIC_COPY_IN_USER_FORMAT_KEY,
				   GINT_TO_POINTER(i));

		g_signal_connect(menuitem, "activate",
				 G_CALLBACK(cb_copy_in_user_format_menuitem_activate),
				 kztab);

		gtk_menu_shell_append(GTK_MENU_SHELL(copy_in_user_format_submenu),
				      menuitem);
		gtk_widget_show(menuitem);

		g_free(title);
	}
}
